/*===========================================================

	the firmware of USB keyboard adapter Apple II plus
	(c) 2010 Koichi NISHIDA

===========================================================*/

/*
hardware information:

use ATTINY2313 AVR.
connect 27MHz (overclock...) crystal to the AVR.
supply 3.3V power.
fuse setting : LOW 11011110

connection:
	PD0:	connect to USB D+ through 33 ohm register, and pull down with 15k ohm register.
	PD1:	connect to USB D- through 33 ohm register, and pull down with 15k ohm register.
	PB0-6:	ASCII code
	PB7 :	strobe : active HIGH for 12u sec (45 usec when repeating)
*/

/*
This is the firmware of USB keyboard adapter for Apple II plus by Nishida Radio.

Copyright (C) 2010 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

	.include "tn2313def.inc"

	; use ATTINY2313 at 27MHz: 18 clock/bit for USB low speed (1.5 M BPS)

	.equ	TIMER_INTV	= 105	; 1m sec (sending EOP interval)
	.equ	INTR_INTV	= 10	; 10 msec (interrupt transfer interval)
	.equ	SIMUL_PUSH 	= 6
		; simulatenous key pushes

	; work area
	.equ	TCOUNT					= 0x60	; for conting INTR_INTV times
	.equ	TSTATE					= 0x61	; for initializing USB
	.equ	REPT_COUNT				= 0x62	; key repeat counter
	.equ	REPT_FLG				= 0x63	; key repeat flag
	.equ	KEY_NUM					= 0x64  ; total pushed key number
	; 0x65 - 0x67 : free
	.equ	KEY_STAT_PREV			= 0x68	; buffer for previous pushed 6 keys
	.equ	KEY_STAT_NOW			= 0x6e 	; buffer for current pushed 6 keys
	.equ	BUFF					= 0x75	; buffer for USB transfer

	.cseg

	rjmp	VECT_RESET
	rjmp	VECT_INT0
	rjmp	VECT_INT1
	rjmp	VECT_TIMER1_CAPT
	rjmp	VECT_TIMER1_COMPA
	rjmp	VECT_TIMER1_OVF
	rjmp	VECT_TIMER0_OVF
	rjmp	VECT_USART0_RX
	rjmp	VECT_USART0_UDRE
	rjmp	VECT_USART0_TX
	rjmp	VECT_ANALOG_COMP
	rjmp	VECT_PCINT
	rjmp	VECT_TIMER1_COMPB
	rjmp	VECT_TIMER0_COMPA
	rjmp	VECT_TIMER0_COMPB
	rjmp	VECT_USI_START
	rjmp	VECT_USI_OVERFLOW
	rjmp	VECT_EE_READY
	rjmp	VECT_WDT_OVERFLOW

VECT_INT0:
VECT_INT1:
VECT_TIMER1_CAPT:
VECT_TIMER1_COMPA:
VECT_TIMER1_OVF:
VECT_TIMER0_OVF:
VECT_USART0_RX:
VECT_USART0_UDRE:
VECT_USART0_TX:
VECT_ANALOG_COMP:
VECT_PCINT:
VECT_TIMER1_COMPB:
VECT_TIMER0_COMPB:
VECT_USI_START:
VECT_USI_OVERFLOW:
VECT_EE_READY:
VECT_WDT_OVERFLOW:
	reti

	; wait 12usec
WAIT12:
	ldi		r20,108
WAIT12LP:
	dec		r20			; 1
	brne	WAIT12LP	; 2
	ret

	; wait 45usec
WAIT45:
	ldi		r20,202
WAIT45LP:
	nop					; 1
	nop					; 1
	nop					; 1
	dec		r20			; 1
	brne	WAIT12LP	; 2
	ret

	; ===== TIMER =====
	; called at every 1m sec
VECT_TIMER0_COMPA:
	rcall	DO_USB
	reti

	; ===== USB communication =====

	;
	; convert data on the ROM to the send format
	;
	; inputs:
	;   Z = source data pointer
	; outputs:
	;	*Z = converted data
	;
CONV_SEND:
	inc		ZL
	lpm
	mov		r18,r0
	inc		ZL
	subi	YL,-2
	push	YL
	;		r19				; temp
	;		r20				; source counter
	ldi		r21,1			; first data (phase) = 1
	ldi		r22,0			; six counter
	ldi		r23,0			; target counter
	;		r24				; target data
CNV_SND_LOOP1:
	ldi		r20,8
	lpm
	inc		ZL
CNV_SND_LOOP2:
	mov		r19,r0
	lsl		r0
	andi	r19,0x80
	brne	CNV_SND_NO_CHANGE_PHASE
CNV_SND_CHANGE_POL:			; msb = 0, change phase
	ldi		r22,0
	ldi		r19,1
	eor		r21,r19
	rjmp	CNV_SND_MERGE
CNV_SND_NO_CHANGE_PHASE:	; msb = 1, not change phase
	inc		r22
CNV_SND_MERGE:
	lsl		r24
	or		r24,r21
	inc		r23
	cpi		r23,8
	brne	CNV_SND_NO_STORE
	ldi		r23, 0
	st		Y+,r24
CNV_SND_NO_STORE:
	cpi		r22,6
	breq	CNV_SND_CHANGE_POL
CNV_SND_NO6:											
	dec		r20
	brne	CNV_SND_LOOP2
	dec		r18
	brne	CNV_SND_LOOP1
	push	r23
	cpi		r23,0
	breq	CNV_SND_END_END
CNV_SND_END_LOOP:
	lsl		r24
	inc		r23
	cpi		r23,8
	brne	CNV_SND_END_LOOP
	st		Y,r24
CNV_SND_END_END:
	pop		r20
	pop		r21
	mov		r19,YL
	sub		r19,r21
	mov		YL,r21
	subi	YL,2
	st		Y+,r19
	st		Y,r20
	ret

	;
	; send data on the RAM to the USB bus
	;
	; inputs:
	;   Y = send data pointer
	;
SEND:
	ld		r19,Y+
	ld		r20,Y+
	ldi		r22,32			; read counter
	lsl		r20
	lsl		r20
	ldi		r23, 0x02		; write data
	out		PORTD, r23
	ldi		r23, 0x3
	out		DDRD, r23
	lsl		r20				; 1
	lsl		r20				; 1
	lsl		r20				; 1
SEND_LOOP:
	nop						; 1
	nop						; 1
	subi	r22,32			; 1
	brne	SEND_WAIT		; 1/2
	ldi		r22,0			; 1
	ld		r21,Y+			; 2
	dec		r19				; 1
	brmi	SEND_END		; 1/2
	brne	SEND_NO_READ	; 1/2
	mov		r22,r20			; 1
SEND_NO_READ:
	ldi		r23,1			; 1
	sbrc	r21,7			; 1
	ldi		r23,2			; 1
	out		PORTD,r23		; 1
	lsl		r21				; 1
	rjmp	SEND_LOOP		; 2
SEND_WAIT:
	nop						; 1
	nop						; 1
	nop						; 1
	nop						; 1
	rjmp	SEND_NO_READ	; 2
SEND_EOP:
	ldi		r18, 0x02		; write data
	out		PORTD, r18
	ldi		r18, 0x3		; 1
	out		DDRD, r18		; 1
	nop						; 1
	ldi		r19,4			; 1
SEND_EOP_LOOP:
	dec		r19				; 1
	brne	SEND_EOP_LOOP	; 2/1
SEND_END:
	nop						; 1
	nop						; 1
	nop						; 1
	ldi		r19,0			; 1
	out		PORTD,r19		; 1
	ldi		r19,11			; 1
SEND_END_LOOP1:
	dec		r19				; 1
	brne	SEND_END_LOOP1	; 2/1
	nop						; 1
	ldi		r19,2			; 1
	out		PORTD,r19		; 1
	ldi		r19,11			; 1
SEND_END_LOOP2:
	dec		r19				; 1
	brne	SEND_END_LOOP2	; 2/1
	nop						; 1
	ldi		r19, 0x0		; 1
	out		DDRD, r19
	out		PORTD, r19
	ret

	;
	; receive data from USB
	;	input = Z : destination pointer
	;	output = *ZL : received data
	;
RECEIVE:
	push	ZL
	inc		ZL
	ldi		r19, 0
	ldi		r20, 0xff
	ldi		r21, 0xff
RECEIVE_WAIT:
	sbic	PIND, 1
	rjmp	RECEIVE_WAIT
RECEIVE_LOOP:
	in		r18, PIND			; 1
	andi	r18, 0x03 			; 1
	breq	RECEIVE_END_LOOP	; 1
	cp		r18,r20				; 1
	mov		r20,r18				; 1
	breq	RECEIVE_EQUAL		; 1/2
RECEIVE_NOT_EQUAL:
	andi	r21,0x3				; 1
	cp		r18,r21				; 1
	brne	RECEIVE_LONGER		; 1/2
RECEIVE_SHORTER:
	ror		r18					; 1
	ror		r22					; 1
	subi	r19,32				; 1
	in		r21,PIND			; 1
	brne	RECEIVE_NO_STORE	; 1/2
	st		Z+,r22				; 2
	rjmp	RECEIVE_LOOP		; 2
RECEIVE_NO_STORE_L:
	nop							; 1
RECEIVE_NO_STORE:
	rjmp	RECEIVE_LOOP		; 2
RECEIVE_EQUAL:
	nop							; 1
	nop							; 1
RECEIVE_LONGER:
	ror		r18					; 1
	ror		r22					; 1
	subi	r19,32				; 1
	in		r21,PIND			; 1
	brne	RECEIVE_NO_STORE_L	; 1/2
	st		Z+,r22				; 2
	rjmp	RECEIVE_LOOP		; 2
RECEIVE_END_LOOP:
	sbis	PIND, 1
	rjmp	RECEIVE_END_LOOP
	mov		r18,ZL
	subi	r18,LOW(BUFF)
	dec		r18
	pop		ZL
	st		Z,r18
	ret

	;
	; convert read data to actual data
	;   input = Y : source pointer
	;	        Z : destination pointer
	;   output = *Z : converted data
	;
CONV_RECEIVE:
	push	r18
	ld		r18,Y+
	ldi		r19, 1				; source counter
	ldi		r20, 8				; destination counter
	ldi		r23, 0xff			; previous value
	ldi		r24, 0				; NZRI count
	ldi		r25, 0				; result value
								; r21 : source value
								; r22 : temprorary
CONV_RCV_LOOP:
	dec		r19
	brne	CONV_RCV_NO_LOAD
	dec 	r18
	breq	CONV_RCV_END
	ldi		r19,8
	ld		r21, Y+
CONV_RCV_NO_LOAD:
	cpi		r24,6
	brne	CONV_RCV_UNDER6
	mov		r22,r21
	ror		r21
	andi	r22,1
	mov		r23,r22
	ldi		r24,0
	rjmp	CONV_RCV_LOOP
CONV_RCV_UNDER6:
	mov		r22,r21
	ror		r21
	andi	r22,1
	cp		r23,r22
	brne	CONV_RCV_NE
CONV_RCV_EQ:
	inc		r24
	sec
	rjmp	CONV_RCV_MERGE
CONV_RCV_NE:
	ldi		r24,0
	clc
CONV_RCV_MERGE:
	mov		r23,r22
	ror		r25
	dec		r20
	brne	CONV_RCV_LOOP
	ldi		r20,8
	st		Z+, r25
	rjmp	CONV_RCV_LOOP
CONV_RCV_END:
	pop		r18
	ret

	; ===== set address =====
SET_ADDRESS:
	; setup stage

	ldi		ZH, HIGH(PKT_SETUP_0*2)
	ldi		ZL, LOW(PKT_SETUP_0*2)
	ldi		YL,LOW(BUFF)
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_SET_ADDRESS*2)
	ldi		ZL, LOW(PKT_SET_ADDRESS*2)
	ldi		YL,LOW(BUFF)+6
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)
	rcall	SEND

	ldi		YL,LOW(BUFF)+6	
	rcall	SEND

	ldi		ZL, LOW(BUFF)	;	should be ack
	rcall	RECEIVE

	; no data stage

	; status stage
SET_ADDRESS_NAK:
	ldi		ZH, HIGH(PKT_IN_0_0*2)
	ldi		ZL, LOW(PKT_IN_0_0*2)
	ldi		YL,LOW(BUFF)
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_ACK*2)
	ldi		ZL, LOW(PKT_ACK*2)
	ldi		YL,LOW(BUFF+14)	
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)
	rcall	SEND

	ldi		ZL, LOW(BUFF)
	rcall	RECEIVE

	lds		r18,BUFF+2
	andi	r18,0x3
	breq	SET_ADDRESS_NAK
	
	; ack
	ldi		YL,LOW(BUFF+14)	
	rcall	SEND

	ldi		ZL,LOW(INIT_RETURN)
	ldi		ZH,HIGH(INIT_RETURN)
	ijmp

	; ===== set configuration =====
SET_CONFIGURATION:
	; setup stage
	ldi		ZH, HIGH(PKT_SETUP_1*2)
	ldi		ZL, LOW(PKT_SETUP_1*2)
	ldi		YL,LOW(BUFF)	
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_SET_CONFIGURATION*2)
	ldi		ZL, LOW(PKT_SET_CONFIGURATION*2)
	ldi		YL,LOW(BUFF)+6
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)	
	rcall	SEND

	ldi		YL,LOW(BUFF)+6
	rcall	SEND

	ldi		ZL, LOW(BUFF)
	rcall	RECEIVE

	; no data stage

	; status stage
SET_CONF_NAK:
	ldi		ZH, HIGH(PKT_IN_1_0*2)
	ldi		ZL, LOW(PKT_IN_1_0*2)	
	ldi		YL,LOW(BUFF)	
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_ACK*2)
	ldi		ZL, LOW(PKT_ACK*2)
	ldi		YL,LOW(BUFF+14)
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)	; pkt in
	rcall	SEND

	ldi		ZL, LOW(BUFF)
	rcall	RECEIVE

	lds		r18,BUFF+2
	andi	r18,0x3
	breq	SET_CONF_NAK

	ldi		YL,LOW(BUFF+14)	; ack
	rcall	SEND

	ldi		ZL,LOW(INIT_RETURN)
	ldi		ZH,HIGH(INIT_RETURN)
	ijmp

	; ===== interrupt transfer =====
INTR_TRANSFER:
	lds		r18,REPT_FLG
	cpi		r18,0
	breq	NO_REPEATING

	; if no keys are pushed, don't repeat
	lds		r18,KEY_NUM
	cpi		r18,2
	brcs	NO_REPEATING

	lds		r18,REPT_COUNT
	cpi		r18,7
	breq	REPT_RESET
	inc		r18
	sts		REPT_COUNT,r18
	ret
REPT_RESET:
	ldi		r18,0
	sts		REPT_COUNT,r18

	; now repeating
	; 45 us STB
	sbi		PORTB,7
	rcall	wait45
	cbi		PORTB,7

NO_REPEATING:
	ldi		ZH, HIGH(PKT_IN_1_1*2)
	ldi		ZL, LOW(PKT_IN_1_1*2)
	ldi		YL,LOW(BUFF)

	rcall	CONV_SEND
	ldi		ZH, HIGH(PKT_ACK*2)
	ldi		ZL, LOW(PKT_ACK*2)
	ldi		YL,LOW(BUFF+17)
	rcall	CONV_SEND
	ldi		YL,LOW(BUFF)
	rcall	SEND
	ldi		r18,0
	sts		(BUFF+5),r18
	ldi		ZL, LOW(BUFF+4)	; destination
	rcall	RECEIVE
	; ack
	ldi		YL,LOW(BUFF+17)	
	rcall	SEND

	; return to main if nak, stall nor not yet
	lds		r18,BUFF+6
	andi	r18,3
	brne	NO_EXIT
	ret
NO_EXIT:
	ldi		YL, LOW(BUFF+4)	; source
	ldi		ZL, LOW(BUFF+4)	; destination
	rcall	CONV_RECEIVE

	; BUFF+6 ... modifires BUFF+8 - +13 ... keys

	; modifier keys

	lds		r18,BUFF+6

	ldi		r24,0
	ldi		r25,0

	sbrc	r18,0	; LEFT CTRL	
	ori		r24,0b10000000		; == CTRL ==
	sbrc	r18,1	; LEFT SHIFT	
	ori		r25,0b00000001		; == SHIFT ==
	sbrc	r18,4	; RIGHT CTRL
	ori		r24,0b10000000		; == CTRL ==
	sbrc	r18,5	; RIGHT SHIFT
	ori		r25,0b00000001		; == SHIFT ==

	; normal keys	
	
	ldi		r20,0
	sts		KEY_NUM,r20

	ldi		XL,LOW(KEY_STAT_NOW)
	ldi		YL,LOW(BUFF+8)
	ldi		r19, SIMUL_PUSH
KEYLOOP6:
	ld		r18,Y+
	cpi		r18,0
	breq	NOT_PUSHED
	lds		r20,KEY_NUM	
	inc		r20
	sts		KEY_NUM,r20
NOT_PUSHED:
	cpi		r18,0x87
	breq	SPECIAL_BACKSLA
	sbrc	r18,7
	ldi		r18,0
SPECIAL_BACKSLA:
	st		X+,r18
	dec		r19
	brne	KEYLOOP6

	; generate PUSH event
	ldi		r19, SIMUL_PUSH
	ldi		XL,LOW(KEY_STAT_NOW)
PUSH_LOOP:

	ld		r18,X+
	ldi		YL,LOW(KEY_STAT_PREV)
	rcall	EXISTS
	brcs	PUSH_EXIST

	; key pushed !
	; pushed key = r18

	; REPT KEY = F11
	cpi		r18,0x44
	brne	NOT_REPT_ON
	ldi		r22,1
	sts		REPT_FLG,r22
	ldi		r22,0
	sts		REPT_COUNT,r22
	rjmp	PUSH_EXIST
NOT_REPT_ON:

	; RESET !
	cpi		r18,0x45
	brne	NOT_RESET
	cbi		PORTD,4
	sbi		DDRD,4
; wait 12 msec = 324 clock
	ldi		r20,108
WAIT_LOOP2:
	dec		r20			; 1
	brne	WAIT_LOOP2	; 2
	cbi		DDRD,4
	rjmp	PUSH_EXIST
NOT_RESET:

	cpi		r18,0x87	; \_ key
	brne	NOT_BACKSLASH
	ldi		r18,0xdc	; \
	sbrc	r25,0
	ldi		r18,0xdf	; _
	rjmp	PUT_BACKSLASH
NOT_BACKSLASH:
	cpi		r18,0		; non effective keys...
	breq	PUSH_EXIST
	ldi		ZL,LOW(KEYMAP*2)
	ldi		ZH,HIGH(KEYMAP*2)
	add		ZL,r24
	ldi		r23,0
	adc		ZH,r23
	add		ZL,r18
	adc		ZH,r25
	lpm		r18,Z
PUT_BACKSLASH:
	out		PORTB,r18
	rcall	wait12
	cbi		PORTB,7
	
PUSH_EXIST:
	dec		r19
	brne	PUSH_LOOP

REL:
	; generate release event
	ldi		r19,SIMUL_PUSH
	ldi		ZL,LOW(KEY_STAT_PREV)
REL_LOOP:
	ld		r18,Z+
	ldi		YL,LOW(KEY_STAT_NOW)
	rcall	EXISTS
	brcs	REL_EXIST

	; key released !

	; REPT KEY = F11
	cpi		r18,0x44
	brne	NOT_REPT_OFF
	ldi		r22,0
	sts		REPT_FLG,r22
NOT_REPT_OFF:

REL_EXIST:
	dec		r19
	brne	REL_LOOP

	; key status update
	ldi		r19,SIMUL_PUSH
	ldi		YL,LOW(KEY_STAT_NOW)
	ldi		ZL,LOW(KEY_STAT_PREV)
STAT_UPDATE_LOOP:
	ld		r18,Y+
	st		Z+,r18
	dec		r19
	brne	STAT_UPDATE_LOOP
	ret

	; examine whether the key exists in a key status list or not
EXISTS:
	ldi		r21, SIMUL_PUSH
EXISTS_LOOP:
	ld		r20,Y+
	cp		r18,r20
	brne	EXISTS_NEXT
	sec
	ret
EXISTS_NEXT:
	dec		r21
	brne	EXISTS_LOOP
	clc
	ret

	; clear BUFF for USB transfer
CLRBUFF:
	ldi		ZL,LOW(BUFF)
	ldi		r19,20
	ldi		r18,0x0
CBUFLP:
	st		Z+,r18
	dec		r19
	brne	CBUFLP
	ret

	; clear key status
CLRSTAT:
	ldi		ZL,LOW(KEY_STAT_PREV)
	ldi		r19,12
	ldi		r18,0x0
CSTLP:
	st		Z+,r18
	dec		r19
	brne	CSTLP
	ret

	; ===== MAIN ENTRANCE =====
VECT_RESET:
		
	ldi		r18,0
	out		PORTB,r18
	ldi		r18,0b11111111
	out		DDRB,r18

	ldi		r18, 0x0
	out		PORTD,r18
	out		DDRD, r18

	rcall	CLRBUFF
	rcall	CLRSTAT

	ldi		r18, LOW(RAMEND)
	out		SPL, r18

	; wait for a while ...
	ldi		r18,255
DW1:
	ldi 	r19,255
DW2:
	ldi		r20,128
DW3:
	dec		r20
	brne	DW3
	dec		r19
	brne	DW2
	dec		r18
	brne	DW1

	; init variables
	ldi		r18,0
	sts		TCOUNT,r18
	sts		TSTATE,r18

	; repeat flag
	sts		REPT_FLG,r18

	; current pushed key number
	sts		KEY_NUM,r18

	; set timer as 1msec
	ldi		r18,TIMER_INTV
	out		OCR0A, r18
	ldi		r18,2
	out		TCCR0A, r18
	ldi		r18, 4			; prescale = 256 times
	out		TCCR0B, r18
	ldi		r18,1
	out		TIMSK, r18
	ldi		r18, 0
	out		TCNT0, r18

	sei

	;===== MAIN LOOP =====
MAIN_LOOP:
	rjmp	MAIN_LOOP

	; USB main routine
	;   called from timer
DO_USB:
	; every 1ms here
	lds		r18,TSTATE
	cpi		r18,205
	brcs	NOT_SEND_EOP
	rcall	SEND_EOP
	lds		r18,TSTATE
NOT_SEND_EOP:
	cpi		r18,217
	brne	INITIALIZE
	lds		r18,TCOUNT
	inc		r18
	sts		TCOUNT,r18
	cpi		r18,INTR_INTV
	breq	TIM_CONT
	ret
TIM_CONT:
	; every INTR_INTV msec here
	ldi		r18,0
	sts		TCOUNT,r18
	rjmp	INTR_TRANSFER	; receive DATA from USB
INITIALIZE:
	cli
	cpi		r18,200
	brne	INIT1
	ldi		r18, 0x3
	out		DDRD, r18
	ldi		r18, 0x00
	out		PORTD, r18
	rjmp	INIT_RETURN
INIT1:cpi	r18,204
	brne	INIT2
	ldi		r18, 0x0
	out		DDRD, r18
	ldi		r18, 0x00
	out		PORTD, r18
	rjmp	INIT_RETURN
INIT2:cpi	r18,214
	brne	INIT3
;	ldi		ZL,LOW(GET_REPORT)
;	ldi		ZH,HIGH(GET_REPORT)
;	ijmp
	rjmp	INIT_RETURN
INIT3:cpi		r18,215
	brne	INIT4
	ldi		ZL,LOW(SET_ADDRESS)
	ldi		ZH,HIGH(SET_ADDRESS)
	ijmp
INIT4:cpi		r18,216
	brne	INIT_RETURN
	ldi		ZL,LOW(SET_CONFIGURATION)
	ldi		ZH,HIGH(SET_CONFIGURATION)
	ijmp
INIT_RETURN:
	; state transition
	lds		r18,TSTATE
	inc		r18
	sts		TSTATE,r18
	sei
USB_EXIT:
	ret

	.org	0x400-(0x80/2)*4-(86/2)

	; ===== data on ROM =====
	; USB packets
PKT_SOF:
	.db		0,4
	.db		0b00000001,0b10100101	;	SYNC, SOF
	.db		0b00000000,0b00001000	;
PKT_ACK:
	.db		0,2
	.db		0b00000001,0b01001011	;	SYNC, ACK
PKT_SETUP_0:
	.db		0,4
	.db		0b00000001,0b10110100	;	SYNC, SETUP
	.db		0b00000000,0b00001000	;	ADDR = 0, ENDP = 0, CRC5
PKT_SET_ADDRESS:
	.db		0,12
	.db		0b00000001,0b11000011	;	SYNC, DATA0
	.db		0b00000000,0b10100000	;	OUT | Standard Request, SET_ADDRESS(5)
	.db		0b10000000,0b00000000	;	wValue = 1
	.db		0b00000000,0b00000000
	.db		0b00000000,0b00000000
	.db		0b11010111,0b10100100	;	CRC16	
PKT_SETUP_1:
	.db		0,4
	.db		0b00000001,0b10110100	;	SYNC, SETUP
	.db		0b10000000,0b00010111	;	ADDR = 1, ENDP = 0, CRC
PKT_SET_CONFIGURATION:
	.db		0,12
	.db		0b00000001,0b11000011	;	SYNC, DATA0
	.db		0b00000000,0b10010000	;	Out | Standard Request, SET_CONFIGURATION(9)
	.db		0b10000000,0b00000000	;	wValue = 1
	.db		0b00000000,0b00000000
	.db		0b00000000,0b00000000
	.db		0b11100100,0b10100100	;	CRC16
PKT_DUMMY_OUT:
	.db		0,4
	.db		0b00000001,0b11010010	;	SYNC, DATA1
	.db		0b00000000,0b00000000	;	CRC16
PKT_OUT_0_0:
	.db		0,4
	.db		0b00000001,0b10000111	;	SYNC, PID_OUT
	.db		0b00000000,0b00001000	;	ADDR = 0, ENDP = 0, CRC
PKT_IN_0_0:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b00000000,0b00001000	;	ADDR = 0, ENDP = 0, CRC
PKT_IN_0_1:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b00000000,0b00000101	;	ADDR = 0, ENDP = 1, CRC
PKT_IN_1_0:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b10000000,0b00010111	;	ADDR = 1, ENDP = 0, CRC
PKT_IN_1_1:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b10000001,0b00011010	;	ADDR = 1, ENDP = 1, CRC

	; keymaps from USB keycord to II keyboard
KEYMAP:
	; ALONE
	;		   0    1    2    3    4    5    6    7    8    9    a    b    c    d    e    f

	; shita check!!! c1,c2???
	.db		0x00,0x00,0x00,0x00,0xC1,0xC2,0xc3,0xc4,0xc5,0xc6,0xc7,0xc8,0xc9,0xca,0xcb,0xcc
	.db		0xcd,0xce,0xcf,0xd0,0xd1,0xd2,0xd3,0xd4,0xd5,0xd6,0xd7,0xd8,0xd9,0xda,0xb1,0xb2
	.db		0xb3,0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xb0,0x8d,0x9b,0x00,0x00,0xa0,0xad,0xde,0xc0
	.db		0xdb,0x00,0xdd,0xbb,0xba,0x00,0xac,0xae,0xaf,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x95
	.db		0x88,0x00,0x00,0x00,0xaf,0xaa,0xad,0xab,0x8d,0xb1,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7
	.db		0xb8,0xb9,0xb0,0xae,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00

	; CONTROL
	.db		0x00,0x00,0x00,0x00,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8a,0x8b,0x8c
	.db		0x8d,0x8e,0x8f,0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9a,0xb1,0xb2
	.db		0xb3,0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xb0,0x8d,0x9b,0x00,0x00,0xa0,0xad,0xde,0xc0
	.db		0xdb,0x00,0xdd,0xbb,0xba,0x00,0xac,0xae,0xaf,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x95
	.db		0x88,0x00,0x00,0x00,0xaf,0xaa,0xad,0xab,0x8d,0xb1,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7
	.db		0xb8,0xb9,0xb0,0xae,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00

	; SHIFT
	.db		0x00,0x00,0x00,0x00,0xc1,0xc2,0xc3,0xc4,0xc5,0xc6,0xc7,0xc8,0xc9,0xca,0xcb,0xcc
	.db		0xdd,0xde,0xcf,0xc0,0xd1,0xd2,0xd3,0xd4,0xd5,0xd6,0xd7,0xd8,0xd9,0xda,0xa1,0xa2
	.db		0xa3,0xa4,0xa5,0xa6,0xa7,0xa8,0xa9,0xb0,0x8d,0x9b,0x00,0x00,0xa0,0xbd,0x00,0x00
	.db		0x00,0x00,0x00,0xab,0xaa,0x00,0xbc,0xbe,0xbf,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x95
	.db		0x88,0x00,0x00,0x00,0xaf,0xaa,0xad,0xab,0x8d,0xb1,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7
	.db		0xb8,0xb9,0xb0,0xae,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00

	; BOTH
	.db		0x00,0x00,0x00,0x00,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8a,0x8b,0x8c
	.db		0x9d,0x9e,0x8f,0x80,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9a,0xa1,0xa2
	.db		0xa3,0xa4,0xa5,0xa6,0xa7,0xa8,0xa9,0xb0,0x8d,0x9b,0x00,0x00,0xa0,0xbd,0x00,0x00
	.db		0x00,0x00,0x00,0xab,0xaa,0x00,0xbc,0xbe,0xbf,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x95
	.db		0x88,0x00,0x00,0x00,0xaf,0xaa,0xad,0xab,0x8d,0xb1,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7
	.db		0xb8,0xb9,0xb0,0xae,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
